package edu.byu.cs240.bookclub.database;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;

import java.util.HashSet;
import java.util.Set;

import edu.byu.cs240.bookclub.model.Book;
import edu.byu.cs240.bookclub.model.Genre;

/**
 * The data access object for the database
 */
public class BookDAO {

    private SQLiteDatabase db;

    public BookDAO(SQLiteDatabase db) {
        this.db = db;
    }

    public Set<Book> getBooksByAuthor(String author) {
        final String SQL = "select id, title, author, genre " +
                            "from book " +
                            "where author = ?";

        Set<Book> result = new HashSet<>();

        Cursor cursor = db.rawQuery(SQL, new String[]{author});
        try {
            cursor.moveToFirst();
            while (!cursor.isAfterLast()) {
                Book book = new Book();

                book.setID(cursor.getLong(0));
                book.setTitle(cursor.getString(1));
                book.setAuthor(cursor.getString(2));
                book.setGenre(Genre.valueOf(cursor.getString(3)));

                result.add(book);

                cursor.moveToNext();
            }
        }
        finally {
            cursor.close();
        }

        return result;
    }

    public boolean addBook(Book book) {
        ContentValues values = new ContentValues();
        values.put("title", book.getTitle());
        values.put("author", book.getAuthor());
        values.put("genre", book.getGenre().toString());

        long id = db.insert("book", null, values);
        if (id >= 0) {
            book.setID(id);
            return true;
        }
        else {
            return false;
        }
    }

    public boolean updateBook(Book book) {
        ContentValues values = new ContentValues();
        values.put("title", book.getTitle());
        values.put("author", book.getAuthor());
        values.put("genre", book.getGenre().toString());

        int rows = db.update("book", values, "id = " + book.getID(), null);
        return (rows == 1);
    }

    public boolean deleteBook(Book book) {
        int rows = db.delete("book", "id = " + book.getID(), null);
        if (rows == 1) {
            book.setID(-1);
            return true;
        }
        else {
            return false;
        }
    }
}
